@@ -1033,6 +1033,12 @@ ixhash_to_bson(bson_t * bson, SV *sv, AV *ids, stackette *stack, int is_insert)
 
     str = SvPVutf8(*k, len);
     containsNullChar(str,len);
+
+    /* if we've already added the oid field, continue */
+    if (ids && strcmp(str, "_id") == 0) {
+        continue;
+    }
+
     append_sv(bson, str, *v, stack, is_insert);
   }
 
@@ -1114,14 +1120,14 @@ append_sv (bson_t * bson, const char * in_key, SV *sv, stackette *stack, int is_
     if (SvGMAGICAL(sv)) {
       mg_get(sv);
     }
-    else {
+  }
+
+  if (!SvOK(sv)) {
       bson_append_null(bson, key, -1);
       if (in_key != key) Safefree((char *)key);
       return;
-    }
   }
-
-  if (SvROK (sv)) {
+  else if (SvROK(sv)) {
     if (sv_isobject (sv)) {
       /* OIDs */
       if (sv_derived_from (sv, "MongoDB::OID")) {
@@ -1657,6 +1663,11 @@ perl_mongo_sv_to_bson (bson_t * bson, SV *sv, AV *ids) {
 
         str = SvPVutf8(*key, len);
 
+        /* if we've already added the oid field, continue */
+        if (ids && strcmp(str, "_id") == 0) {
+            continue;
+        }
+
         append_sv (bson, str, *val, EMPTY_STACK, is_insert);
       }
     }
